/* ==================================================================================================================
tabs.css
==================================================================================================================
*/

.tabs, .tabs2, .tabs3 {
    background-color: #c8c7d2;
    width: 100%;
    z-index: 101;
    font-size: 13px;
    font-family: 'Lucida Grande',Geneva,Helvetica,Arial,sans-serif;
}

.tabs2 {
    font-size: 10px;
}
.tabs3 {
    font-size: 9px;
}

.tablist {
    margin: 0;
    padding: 0;
    display: table;
}

.tablist li {
    float: left;
    display: table-cell;
    line-height: 20px;
    list-style: none;
}

.tablist a {
    display: block;
    padding: 0 20px;
    font-weight: bold;
    color: #25242D;
    text-shadow: 0px 1px 1px rgba(255, 255, 255, 0.9);
    text-decoration: none;
    outline: none;
}

.tabs3 .tablist a {
    padding: 0 10px;
}

.tablist a:hover {
    background-color: #88869e;
    color: #fff;
    text-shadow: 0px 1px 1px rgba(0, 0, 0, 1.0);
    text-decoration: none;
}

.tablist li.current a {
    background-color: #444253;
    color: #fff;
    text-shadow: 0px 1px 1px rgba(0, 0, 0, 1.0);
}

/* ==================================================================================================================
search.css
==================================================================================================================
*/
/*---------------- Search Box */

#FSearchBox {
    float: left;
}

#MSearchBox {
    margin: 0px;
    white-space : nowrap;
    position: absolute;
    float: none;
    display: inline;
    right: 0px;
    width: 170px;
    z-index: 102;
    background-color: white;
}

#MSearchBox .left
{
    display:block;
    position:absolute;
    left:10px;
    width:130px;
    height:18px;
    border: 1px solid gray;
    background-color:#FFF;
    border-radius:8px 0px 0px 8px;
    border-right: 0;
}

#MSearchSelect {
    display:block;
    position:absolute;
    width:20px;
    height:19px;
}

.left #MSearchSelect {
    left:4px;
}

.right #MSearchSelect {
    right:5px;
}

#MSearchField {
    display:block;
    position:absolute;
    height:17px;
    border: none;
    width:111px;
    margin-left:20px;
    padding-left:4px;
    color: #909090;
    outline: none;
    font: 9pt Arial, Verdana, sans-serif;
    background-color: transparent;
}

#FSearchBox #MSearchField {
    margin-left:15px;
}

img {
  border-style: none;
}

#MSearchCloseImg {
  width: 11px;
  height: 11px;
}

#MSearchBox .right {
    display:block;
    position:absolute;
    right:10px;
    top:0px;
    width:20px;
    height:18px;
    border: 1px solid gray;
    background-color:#FFF;
    border-radius:0px 8px 8px 0px;
    border-left: 0;
}

#MSearchClose {
    display: none;
    position: absolute;
    top: 4px;
    background : none;
    border: none;
    margin: 0px 4px 0px 0px;
    padding: 0px 0px;
    outline: none;
}

.left #MSearchClose {
    left: 6px;
}

.right #MSearchClose {
    right: 2px;
}

.MSearchBoxActive #MSearchField {
    color: #000000;
}

/*---------------- Search filter selection */

#MSearchSelectWindow {
    display: none;
    position: absolute;
    left: 0; top: 0;
    border: 1px solid #8F8DA3;
    background-color: #F9F8FA;
    z-index: 1;
    padding-top: 4px;
    padding-bottom: 4px;
    -webkit-box-shadow: 5px 5px 5px rgba(0, 0, 0, 0.15);
}

.SelectItem {
    font: 8pt Arial, Verdana, sans-serif;
    padding-left:  2px;
    padding-right: 12px;
    border: 0px;
}

span.SelectionMark {
    margin-right: 4px;
    font-family: monospace;
    outline-style: none;
    text-decoration: none;
}

a.SelectItem {
    display: block;
    outline-style: none;
    color: #000000; 
    text-decoration: none;
    padding-left:   6px;
    padding-right: 12px;
}

a.SelectItem:focus,
a.SelectItem:active {
    color: #000000; 
    outline-style: none;
    text-decoration: none;
}

a.SelectItem:hover {
    color: #FFFFFF;
    background-color: #403F4F;
    outline-style: none;
    text-decoration: none;
    cursor: pointer;
    display: block;
}

/*---------------- Search results window */

iframe#MSearchResults {
    width: 60ex;
    height: 15em;
}

#MSearchResultsWindow {
    display: none;
    position: absolute;
    left: 0; top: 0;
    border: 1px solid #000;
    background-color: #EDECF0;
}

/* ----------------------------------- */


#SRIndex {
    clear:both; 
    padding-bottom: 15px;
}

.SREntry {
    font-size: 10pt;
    padding-left: 1ex;
}

.SRPage .SREntry {
    font-size: 8pt;
    padding: 1px 5px;
}

body.SRPage {
    margin: 5px 2px;
}

.SRChildren {
    padding-left: 3ex; padding-bottom: .5em 
}

.SRPage .SRChildren {
    display: none;
}

.SRSymbol {
    font-weight: bold; 
    color: #484657;
    font-family: Arial, Verdana, sans-serif;
    text-decoration: none;
    outline: none;
}

a.SRScope {
    display: block;
    color: #484657; 
    font-family: Arial, Verdana, sans-serif;
    text-decoration: none;
    outline: none;
}

a.SRSymbol:focus, a.SRSymbol:active,
a.SRScope:focus, a.SRScope:active {
    text-decoration: underline;
}

span.SRScope {
    padding-left: 4px;
}

.SRPage .SRStatus {
    padding: 2px 5px;
    font-size: 8pt;
    font-style: italic;
}

.SRResult {
    display: none;
}

DIV.searchresults {
    margin-left: 10px;
    margin-right: 10px;
}

/*---------------- External search page results */

.searchresult {
    background-color: #EFEFF2;
}

.pages b {
   color: white;
   padding: 5px 5px 3px 5px;
   background-image: url("../tab_a.png");
   background-repeat: repeat-x;
   text-shadow: 0 1px 1px #000000;
}

.pages {
    line-height: 17px;
    margin-left: 4px;
    text-decoration: none;
}

.hl {
    font-weight: bold;
}

#searchresults {
    margin-bottom: 20px;
}

.searchpages {
    margin-top: 10px;
}


/* ==================================================================================================================
doxygen.css
==================================================================================================================
*/

/* The standard CSS for doxygen 1.8.8 */

body, table, div, p, dl {
	font: 13px arial,sans-serif;
}

@media screen and (max-width: 700px){
  body, table, div, p, dl {
    font: 12px arial,sans-serif !important;
  }
}

@media screen and (max-width: 650px){
  body, table, div, p, dl {
    font: 11px arial,sans-serif !important;
  }
}

/* @group Heading Levels */

h1.groupheader {
	font-size: 150%;
}

.title {
	font: 400 14px/28px Roboto,sans-serif;
	font-size: 150%;
	font-weight: bold;
	margin: 10px 2px;
}

h2.groupheader {
	border-bottom: 1px solid #87859D;
	color: #363542;
	font-size: 150%;
	font-weight: normal;
	margin-top: 1.75em;
	padding-top: 8px;
	padding-bottom: 4px;
	width: 100%;
}

h3.groupheader {
	font-size: 100%;
}

h1, h2, h3, h4, h5, h6 {
	-webkit-transition: text-shadow 0.5s linear;
	-moz-transition: text-shadow 0.5s linear;
	-ms-transition: text-shadow 0.5s linear;
	-o-transition: text-shadow 0.5s linear;
	transition: text-shadow 0.5s linear;
	margin-right: 15px;
}

h1.glow, h2.glow, h3.glow, h4.glow, h5.glow, h6.glow {
	text-shadow: 0 0 15px cyan;
}

dt {
	font-weight: bold;
}

div.multicol {
	-moz-column-gap: 1em;
	-webkit-column-gap: 1em;
	-moz-column-count: 3;
	-webkit-column-count: 3;
}

p.startli, p.startdd {
	margin-top: 2px;
}

p.starttd {
	margin-top: 0px;
}

p.endli {
	margin-bottom: 0px;
}

p.enddd {
	margin-bottom: 4px;
}

p.endtd {
	margin-bottom: 2px;
}

/* @end */

caption {
	font-weight: bold;
}

span.legend {
  font-size: 70%;
  text-align: center;
}

h3.version {
  font-size: 90%;
  text-align: center;
}

div.qindex, div.navtab{
	background-color: #EAEAEE;
	border: 1px solid #A2A0B2;
	text-align: center;
}

div.qindex, div.navpath {
	width: 100%;
	line-height: 140%;
}

div.navtab {
	margin-right: 15px;
}

/* @group Link Styling */

a {
	color: #403F4F;
	font-weight: normal;
	text-decoration: none;
}

.contents a:visited {
	color: #4F4D60;
}

a:hover {
	text-decoration: underline;
}

a.el {
	font-weight: bold;
}

/* @end */

div.ah {
	background-color: black;
	font-weight: bold;
	color: #ffffff;
	margin-bottom: 3px;
	margin-top: 3px;
	padding: 0.2em;
	border: solid thin #333;
}

body {
	background-color: white;
	color: black;
  margin: 0;
}

div.contents {
	margin-top: 10px;
	margin-left: 12px;
	margin-right: 8px;
}

td.indexkey {
	background-color: #EAEAEE;
	font-weight: bold;
	border: 1px solid #C2C1CD;
	margin: 2px 0px 2px 0;
	padding: 2px 10px;
  white-space: nowrap;
  vertical-align: top;
}

td.indexvalue {
	background-color: #EAEAEE;
	border: 1px solid #C2C1CD;
	padding: 2px 10px;
	margin: 2px 0px;
}

tr.memlist {
	background-color: #EDECF0;
}

p.formulaDsp {
	text-align: center;
}

img.formulaDsp {
	
}

img.formulaInl {
	vertical-align: middle;
}

div.center {
	text-align: center;
  margin-top: 0px;
  margin-bottom: 0px;
  padding: 0px;
}

div.center img {
	border: 0px;
}

address.footer {
	text-align: right;
	padding-right: 12px;
}

img.footer {
	border: 0px;
	vertical-align: middle;
}

td.tiny {
	font-size: 75%;
}

.dirtab {
	padding: 4px;
	border-collapse: collapse;
	border: 1px solid #A2A0B2;
}

th.dirtab {
	background: #EAEAEE;
	font-weight: bold;
}

hr {
	height: 0px;
	border: none;
	border-top: 1px solid #545267;
}

hr.footer {
	height: 1px;
}

/* @group Member Descriptions */

table.memberdecls {
	border-spacing: 0px;
	padding: 0px;
}

.memberdecls td, .fieldtable tr {
	-webkit-transition-property: background-color, box-shadow;
	-webkit-transition-duration: 0.5s;
	-moz-transition-property: background-color, box-shadow;
	-moz-transition-duration: 0.5s;
	-ms-transition-property: background-color, box-shadow;
	-ms-transition-duration: 0.5s;
	-o-transition-property: background-color, box-shadow;
	-o-transition-duration: 0.5s;
	transition-property: background-color, box-shadow;
	transition-duration: 0.5s;
}

.memberdecls td.glow, .fieldtable tr.glow {
	background-color: cyan;
	box-shadow: 0 0 15px cyan;
}

.mdescLeft, .mdescRight,
.memItemLeft, .memItemRight,
.memTemplItemLeft, .memTemplItemRight, .memTemplParams {
	background-color: transparent;
	border: none;
	margin: 4px;
	padding: 1px 0 0 8px;
}

.mdescLeft, .mdescRight {
	padding: 0px 8px 4px 8px;
	color: #555;
}

@media screen and (min-width: 500px){
  .memItemLeft, .memTemplItemLeft {
    white-space: nowrap;
  }
}

.memItemLeft, .memTemplItemLeft {
  text-align: right;
  vertical-align: top;
}

.memItemRight {
	width: 100%;
  vertical-align: bottom;
}

.memTemplParams {
	color: #4F4D60;
  white-space: nowrap;
	font-size: 80%;
}

/* @end */

/* @group Member Details */

/* Styles for detailed member documentation */

.memtemplate {
	font-size: 80%;
	color: #4F4D60;
	font-weight: normal;
	margin-left: 9px;
}

.memnav {
	background-color: #EAEAEE;
	border: 1px solid #A2A0B2;
	text-align: center;
	margin: 2px;
	margin-right: 15px;
	padding: 2px;
}

.mempage {
	width: 100%;
}

.memitem {
	padding: 0;
	margin-bottom: 8px;
	margin-right: 5px;
  -webkit-transition: box-shadow 0.5s linear;
  -moz-transition: box-shadow 0.5s linear;
  -ms-transition: box-shadow 0.5s linear;
  -o-transition: box-shadow 0.5s linear;
  transition: box-shadow 0.5s linear;
  display: table !important;
  width: 100%;
}

.memitem.glow {
   box-shadow: 0 0 15px cyan;
}

.memproto, dl.reflist dt {
  border: 1px solid #A6A4B6;
  padding: 6px;
  color: #202027;
  font-weight: bold;
  text-shadow: 0px 1px 1px rgba(255, 255, 255, 0.9);
  background-color: #E1E0E6;
}

.memdoc, dl.reflist dd {
  border-bottom: 1px solid #A6A4B6;      
  border-left: 1px solid #A6A4B6;      
  border-right: 1px solid #A6A4B6; 
  padding: 6px 10px 2px 10px;
  border-top-width: 0;
  background-color: #FFFFFF;
}

dl.reflist dt {
  padding: 5px;
}

dl.reflist dd {
  margin: 0px 0px 10px 0px;
  padding: 5px;
}

.paramname {
	color: #602020;
	white-space: nowrap;
}
.paramname em {
	font-style: normal;
}
.paramname code {
  line-height: 14px;
}

.params, .retval, .exception, .tparams {
  margin-left: 0px;
  padding-left: 0px;
}       

.params .paramname, .retval .paramname {
  font-weight: bold;
  vertical-align: top;
}
  
.params .paramtype {
  font-style: italic;
  vertical-align: top;
}       
  
.params .paramdir {
  font-family: "courier new",courier,monospace;
  vertical-align: top;
}

span.mlabels {
  margin-left: 8px;
  float: right;
}

span.mlabel {
  background-color: #74718D;
  border-top:1px solid #5E5B72;
  border-left:1px solid #5E5B72;
  border-right:1px solid #C2C1CD;
  border-bottom:1px solid #C2C1CD;
	text-shadow: none;
	color: white;
	margin-right: 4px;
	padding: 2px 3px;
	font-size: 7pt;
	white-space: nowrap;
	vertical-align: middle;
}



/* @end */

/* these are for tree view inside a (index) page */

div.directory {
  margin: 10px 0px;
  border-top: 1px solid #9B99AD;
  border-bottom: 1px solid #9B99AD;
  width: 100%;
}

.directory table {
  border-collapse:collapse;
}

.directory td {
  margin: 0px;
  padding: 0px;
	vertical-align: top;
}

.directory td.entry {
  padding-right: 6px;
	padding-top: 3px;
}

@media screen and (min-width: 500px){
  .directory td.entry {
    white-space: nowrap;
  }
}

.directory td.entry a {
  outline:none;
}

.directory td.entry a img {
  border: none;
}

.directory td.desc {
  width: 100%;
  padding-left: 6px;
	padding-right: 6px;
	padding-top: 3px;
	border-left: 1px solid rgba(0,0,0,0.05);
}

.directory tr:nth-child(odd) {
	background-color: #F6F6F8;
}

.directory img {
	vertical-align: -30%;
}

.directory .levels {
  white-space: nowrap;
  width: 100%;
  text-align: right;
  font-size: 9pt;
}

.directory .levels span {
  cursor: pointer;
  padding-left: 2px;
  padding-right: 2px;
	color: #403F4F;
}

.arrow {
    color: #9B99AD;
    -webkit-user-select: none;
    -khtml-user-select: none;
    -moz-user-select: none;
    -ms-user-select: none;
    user-select: none;
    cursor: pointer;
    font-size: 80%;
    display: inline-block;
    width: 16px;
    height: 22px;
}

.icon {
    font-family: Arial, Helvetica;
    font-weight: bold;
    font-size: 12px;
    height: 14px;
    width: 16px;
    display: inline-block;
    background-color: #74718D;
    color: white;
    text-align: center;
    margin-left: 2px;
    margin-right: 2px;
}

.icona {
    width: 24px;
    height: 22px;
    display: inline-block;
}

.iconfopen {
    width: 24px;
    height: 18px;
    margin-bottom: 4px;
    background-image:url('ftv2folderopen.png');
    background-position: 0px -4px;
    background-repeat: repeat-y;
    vertical-align:top;
    display: inline-block;
}

.iconfclosed {
    width: 24px;
    height: 18px;
    margin-bottom: 4px;
    background-image:url('ftv2folderclosed.png');
    background-position: 0px -4px;
    background-repeat: repeat-y;
    vertical-align:top;
    display: inline-block;
}

.icondoc {
    width: 24px;
    height: 18px;
    margin-bottom: 4px;
    background-image:url('ftv2doc.png');
    background-position: 0px -4px;
    background-repeat: repeat-y;
    vertical-align:top;
    display: inline-block;
}

table.directory {
    font: 400 14px Roboto,sans-serif;
}

/* @end */

div.dynheader {
  margin-top: 8px;
	-webkit-touch-callout: none;
	-webkit-user-select: none;
	-khtml-user-select: none;
	-moz-user-select: none;
	-ms-user-select: none;
	user-select: none;
}

address {
	font-style: normal;
	color: #272630;
}

table.doxtable {
	border-collapse:collapse;
  margin-top: 4px;
  margin-bottom: 4px;
}

table.doxtable td, table.doxtable th {
	border: 1px solid #2B2934;
	padding: 3px 7px 2px;
}

table.doxtable th {
	background-color: #383745;
	color: #FFFFFF;
	font-size: 110%;
	padding-bottom: 4px;
	padding-top: 5px;
}

table.fieldtable {
  /*width: 100%;*/
  margin-bottom: 10px;
  border: 1px solid #A6A4B6;
  border-spacing: 0px;
}

.fieldtable td, .fieldtable th {
  padding: 3px 7px 2px;
}

.fieldtable td.fieldtype, .fieldtable td.fieldname {
  white-space: nowrap;
  border-right: 1px solid #A6A4B6;
  border-bottom: 1px solid #A6A4B6;
  vertical-align: top;
}

.fieldtable td.fieldname {
  padding-top: 3px;
}

.fieldtable td.fielddoc {
  border-bottom: 1px solid #A6A4B6;
  /*width: 100%;*/
}

.fieldtable td.fielddoc p:first-child {
  margin-top: 0px;
}       
  
.fieldtable td.fielddoc p:last-child {
  margin-bottom: 2px;
}

.fieldtable tr:last-child td {
  border-bottom: none;
}

.fieldtable th {
  background-color: #E1E0E6;
  font-size: 90%;
  color: #202027;
  padding-bottom: 4px;
  padding-top: 5px;
  text-align:left;
  border-bottom: 1px solid #A6A4B6;
}


.tabsearch {
	top: 0px;
	left: 10px;
	height: 36px;
	background-image: url('tab_b.png');
	z-index: 101;
	overflow: hidden;
	font-size: 13px;
}

.navpath ul
{
	font-size: 11px;
	height:30px;
	line-height:30px;
	color:#8A879F;
	border:solid 1px #C0BFCB;
	overflow:hidden;
	margin:0px;
	padding:0px;
}

.navpath li
{
	list-style-type:none;
	float:left;
	padding-left:10px;
	padding-right:15px;
	background-image:url('bc_s.png');
	background-repeat:no-repeat;
	background-position:right;
	color:#373543;
}

.navpath li.navelem a
{
	height:32px;
	display:block;
	text-decoration: none;
	outline: none;
	color: #25242D;
	font-family: 'Lucida Grande',Geneva,Helvetica,Arial,sans-serif;
	text-shadow: 0px 1px 1px rgba(255, 255, 255, 0.9);
	text-decoration: none;  
}

.navpath li.navelem a:hover
{
	color:#6C6A84;
}

.navpath li.footer
{
  list-style-type:none;
  float:right;
  padding-left:10px;
  padding-right:15px;
  background-image:none;
  background-repeat:no-repeat;
  background-position:right;
  color:#373543;
  font-size: 8pt;
}


div.summary
{
	float: right;
	font-size: 8pt;
	padding-right: 5px;
	width: 50%;
	text-align: right;
}       

div.summary a
{
	white-space: nowrap;
}

div.ingroups
{
	font-size: 8pt;
	width: 50%;
	text-align: left;
}

div.ingroups a
{
	white-space: nowrap;
}

div.header
{
	background-color: #F9F8FA;
	margin:  0px;
	border-bottom: 1px solid #C2C1CD;
}

div.headertitle
{
	padding: 5px 5px 5px 10px;
}

dl
{
  padding: 0 0 0 10px;
}

/* dl.note, dl.warning, dl.attention, dl.pre, dl.post, dl.invariant, dl.deprecated, dl.todo, dl.test, dl.bug */
dl.section
{
	margin-left: 0px;
	padding-left: 0px;
}

dl.note, dl.warning, dl.attention, dl.pre, dl.post, dl.invariant, dl.deprecated, dl.todo, dl.test, dl.bug
{
  margin-left:-7px;
  padding-left: 3px;
  border-left:4px solid;
}
dl.note
{
  border-color: #D0C000;
}

dl.warning, dl.attention
{
  border-color: #FF0000;
}

dl.pre, dl.post, dl.invariant
{
  border-color: #00D000;
}

dl.deprecated
{
  border-color: #505050;
}

dl.todo
{
  border-color: #00C0E0;
}

dl.test
{
  border-color: #3030E0;
}

dl.bug
{
  border-color: #C08050;
}

dl.section dd {
	margin-bottom: 6px;
}


#projectlogo
{
	text-align: center;
	vertical-align: bottom;
	border-collapse: separate;
}
 
#projectlogo img
{ 
	border: 0px none;
}
 
#projectname
{
	font: 300% Tahoma, Arial,sans-serif;
	margin: 0px;
	padding: 2px 0px;
}
    
#projectbrief
{
	font: 120% Tahoma, Arial,sans-serif;
	margin: 0px;
	padding: 0px;
}

#projectnumber
{
	font: 50% Tahoma, Arial,sans-serif;
	margin: 0px;
	padding: 0px;
}

#titlearea
{
	padding: 0px;
	margin: 0px;
	width: 100%;
	border-bottom: 1px solid #5E5B72;
  overflow: hidden;
  position: relative;
}

#titlearea td
{
  padding: 0px;
}

#titlearea table
{
  border-spacing: 0px;
}

.image, .dotgraph, .mscgraph, .diagraph
{
  text-align: center;
}

.caption
{
	font-weight: bold;
}

div.zoom
{
	border: 1px solid #8F8DA3;
}

dl.citelist {
  margin-bottom:50px;
}

dl.citelist dt {
  color:#32313D;
  float:left;
  font-weight:bold;
  margin-right:10px;
  padding:5px;
}

dl.citelist dd {
  margin:2px 0;
  padding:5px 0;
}

div.toc {
  padding: 14px 25px;
  background-color: #F4F4F6;
  border: 1px solid #D6D5DD;
  float: right;
  height: auto;
  margin: 0 20px 10px 10px;
  width: 200px;
}

div.toc li {
  background: url("bdwn.png") no-repeat scroll 0 5px transparent;
  font: 10px/1.2 Verdana,DejaVu Sans,Geneva,sans-serif;
  margin-top: 5px;
  padding-left: 10px;
  padding-top: 2px;
}

div.toc h3 {
  font: bold 12px/1.2 Arial,FreeSans,sans-serif;
	color: #4F4D60;
  border-bottom: 0 none;
  margin: 0;
}

div.toc ul {
  list-style: none outside none;
  border: medium none;
  padding: 0px;
}       

div.toc li.level1 {
  margin-left: 0px;
}

div.toc li.level2 {
  margin-left: 15px;
}

div.toc li.level3 {
  margin-left: 30px;
}

div.toc li.level4 {
  margin-left: 45px;
}

.inherit_header {
  font-weight: bold;
  color: gray;
  cursor: pointer;
	-webkit-touch-callout: none;
	-webkit-user-select: none;
	-khtml-user-select: none;
	-moz-user-select: none;
	-ms-user-select: none;
	user-select: none;
}

.inherit_header td {
  padding: 6px 0px 2px 5px;
}

.inherit {
  display: none;
}

tr.heading h2 {
  margin-top: 12px;
  margin-bottom: 4px;
}

/* tooltip related style info */

.ttc {
  position: absolute;
  display: none;
}

#powerTip {
	cursor: default;
	white-space: nowrap;
	background-color: white;
	border: 1px solid gray;
	box-shadow: 1px 1px 7px gray;
	display: none;
	font-size: smaller;
	max-width: 80%;
	opacity: 0.9;
	padding: 1ex 1em 1em;
	position: absolute;
	z-index: 2147483647;
}

#powerTip div.ttdoc {
  color: grey;
	font-style: italic;
}

#powerTip div.ttname a {
  font-weight: bold;
}

#powerTip div.ttname {
  font-weight: bold;
}

#powerTip div.ttdeci {
  color: #006318;
}

#powerTip div {
  margin: 0px;
  padding: 0px;
  font: 12px/16px Roboto,sans-serif;
}

#powerTip:before, #powerTip:after {
	content: "";
	position: absolute;
	margin: 0px;
}

#powerTip.n:after,  #powerTip.n:before,
#powerTip.s:after,  #powerTip.s:before,
#powerTip.w:after,  #powerTip.w:before,
#powerTip.e:after,  #powerTip.e:before,
#powerTip.ne:after, #powerTip.ne:before,
#powerTip.se:after, #powerTip.se:before,
#powerTip.nw:after, #powerTip.nw:before,
#powerTip.sw:after, #powerTip.sw:before {
	border: solid transparent;
	content: " ";
	height: 0;
	width: 0;
	position: absolute;
}

#powerTip.n:after,  #powerTip.s:after,
#powerTip.w:after,  #powerTip.e:after,
#powerTip.nw:after, #powerTip.ne:after,
#powerTip.sw:after, #powerTip.se:after {
	border-color: rgba(255, 255, 255, 0);
}

#powerTip.n:before,  #powerTip.s:before,
#powerTip.w:before,  #powerTip.e:before,
#powerTip.nw:before, #powerTip.ne:before,
#powerTip.sw:before, #powerTip.se:before {
	border-color: rgba(128, 128, 128, 0);
}

#powerTip.n:after,  #powerTip.n:before,
#powerTip.ne:after, #powerTip.ne:before,
#powerTip.nw:after, #powerTip.nw:before {
	top: 100%;
}

#powerTip.n:after, #powerTip.ne:after, #powerTip.nw:after {
	border-top-color: #ffffff;
	border-width: 10px;
	margin: 0px -10px;
}
#powerTip.n:before {
	border-top-color: #808080;
	border-width: 11px;
	margin: 0px -11px;
}
#powerTip.n:after, #powerTip.n:before {
	left: 50%;
}

#powerTip.nw:after, #powerTip.nw:before {
	right: 14px;
}

#powerTip.ne:after, #powerTip.ne:before {
	left: 14px;
}

#powerTip.s:after,  #powerTip.s:before,
#powerTip.se:after, #powerTip.se:before,
#powerTip.sw:after, #powerTip.sw:before {
	bottom: 100%;
}

#powerTip.s:after, #powerTip.se:after, #powerTip.sw:after {
	border-bottom-color: #ffffff;
	border-width: 10px;
	margin: 0px -10px;
}

#powerTip.s:before, #powerTip.se:before, #powerTip.sw:before {
	border-bottom-color: #808080;
	border-width: 11px;
	margin: 0px -11px;
}

#powerTip.s:after, #powerTip.s:before {
	left: 50%;
}

#powerTip.sw:after, #powerTip.sw:before {
	right: 14px;
}

#powerTip.se:after, #powerTip.se:before {
	left: 14px;
}

#powerTip.e:after, #powerTip.e:before {
	left: 100%;
}
#powerTip.e:after {
	border-left-color: #ffffff;
	border-width: 10px;
	top: 50%;
	margin-top: -10px;
}
#powerTip.e:before {
	border-left-color: #808080;
	border-width: 11px;
	top: 50%;
	margin-top: -11px;
}

#powerTip.w:after, #powerTip.w:before {
	right: 100%;
}
#powerTip.w:after {
	border-right-color: #ffffff;
	border-width: 10px;
	top: 50%;
	margin-top: -10px;
}
#powerTip.w:before {
	border-right-color: #808080;
	border-width: 11px;
	top: 50%;
	margin-top: -11px;
}

@media print
{
  #top { display: none; }
  #side-nav { display: none; }
  #nav-path { display: none; }
  body { overflow:visible; }
  h1, h2, h3, h4, h5, h6 { page-break-after: avoid; }
  .summary { display: none; }
  .memitem { page-break-inside: avoid; }
  #doc-content
  {
    margin-left:0 !important;
    height:auto !important;
    width:auto !important;
    overflow:inherit;
    display:inline;
  }
  #google_translate_element { display: none; }
}

/* ==================================================================================================================
customdoxygen.css
==================================================================================================================
*/

/* Custom Look and feel - Can be edited
----------------------------------- */
#navrow1, #navrow2, #navrow3, #navrow4, #navrow5{
    border-bottom: 1px solid black;
}

.adjust-right {
	margin-left: 30px !important;
	font-size: 1.15em !important;
}

/* Ribbon; Fork me banner; Modified from: https://unindented.org/articles/github-ribbon-using-css-transforms/ 
Note: Parent must have overflow:hidden and position:relative.
*/
.ribbon {
  background-color: #0a0;
  white-space: nowrap;
  position: absolute;
  right: -70px;
  top: 10px;
  -webkit-transform: rotate(22deg);
     -moz-transform: rotate(22deg);
      -ms-transform: rotate(22deg);
       -o-transform: rotate(22deg);
    transform: rotate(22deg);
  -webkit-box-shadow: 0 0 10px #888;
     -moz-box-shadow: 0 0 10px #888;
    box-shadow: 0 0 10px #888;
  z-index: 9999;
}
.ribbon a {
  border: 1px solid #afa;
  color: #fff;
  display: block;
  font: bold 81.25% 'Helvetica Neue', Helvetica, Arial, sans-serif;
  margin: 1px 0;
  padding: 10px 80px;
  text-align: center;
  text-decoration: none;
  text-shadow: 0 0 5px #444;
}

@media print {
  .twitter-timeline {
    display: none !important;
  }
}

@media screen and (max-width: 400px), print {
  #projectlogo {
    display: none !important;
  }
}

/* DOXYGEN Code Styles - These are from doxygen but can be changed to fit the theme
----------------------------------- */

a.qindex {
    font-weight: bold;
}

a.qindexHL {
    font-weight: bold;
    background-color: #9CAFD4;
    color: #ffffff;
    border: 1px double #869DCA;
}

.contents a.qindexHL:visited {
    color: #ffffff;
}

a.code, a.code:visited, a.line, a.line:visited {
    color: #4665A2;
}

a.codeRef, a.codeRef:visited, a.lineRef, a.lineRef:visited {
    color: #4665A2;
}

/* @end */

dl.el {
    margin-left: -1cm;
}

pre.fragment {
    border: 1px solid #C4CFE5;
    background-color: #FBFCFD;
    padding: 4px 6px;
    margin: 4px 8px 4px 2px;
    overflow: auto;
    word-wrap: break-word;
    font-size:  9pt;
    line-height: 125%;
    font-family: monospace, fixed;
    font-size: 105%;
}

div.fragment {
    padding: 4px 6px;
    margin: 4px 8px 4px 2px;
    background-color: #FBFBFC;
    border: 1px solid #C4CFE5;
}

div.line {
    font-family: monospace, fixed;
    font-size: 13px;
    min-height: 13px;
    line-height: 1.0;
    text-wrap: unrestricted;
    white-space: -moz-pre-wrap; /* Moz */
    white-space: -pre-wrap;     /* Opera 4-6 */
    white-space: -o-pre-wrap;   /* Opera 7 */
    white-space: pre-wrap;      /* CSS3  */
    word-wrap: break-word;      /* IE 5.5+ */
    text-indent: -53px;
    padding-left: 53px;
    padding-bottom: 0px;
    margin: 0px;
    -webkit-transition-property: background-color, box-shadow;
    -webkit-transition-duration: 0.5s;
    -moz-transition-property: background-color, box-shadow;
    -moz-transition-duration: 0.5s;
    -ms-transition-property: background-color, box-shadow;
    -ms-transition-duration: 0.5s;
    -o-transition-property: background-color, box-shadow;
    -o-transition-duration: 0.5s;
    transition-property: background-color, box-shadow;
    transition-duration: 0.5s;
}

div.line.glow {
    background-color: cyan;
    box-shadow: 0 0 10px cyan;
}


span.lineno {
    padding-right: 4px;
    text-align: right;
    border-right: 2px solid #0F0;
    background-color: #E8E8E8;
    white-space: pre;
}
span.lineno a {
    background-color: #D8D8D8;
}

span.lineno a:hover {
    background-color: #C8C8C8;
}

div.groupHeader {
    margin-left: 16px;
    margin-top: 12px;
    font-weight: bold;
}

div.groupText {
    margin-left: 16px;
    font-style: italic;
}

/* @group Code Colorization */

span.keyword {
    color: #008000
}

span.keywordtype {
    color: #604020
}

span.keywordflow {
    color: #e08000
}

span.comment {
    color: #800000
}

span.preprocessor {
    color: #806020
}

span.stringliteral {
    color: #002080
}

span.charliteral {
    color: #008080
}

span.vhdldigit {
    color: #ff00ff
}

span.vhdlchar {
    color: #000000
}

span.vhdlkeyword {
    color: #700070
}

span.vhdllogic {
    color: #ff0000
}

blockquote {
    background-color: #F7F8FB;
    border-left: 2px solid #9CAFD4;
    margin: 0 24px 0 4px;
    padding: 0 12px 0 16px;
}

/* ==================================================================================================================
cmdbtns.css
==================================================================================================================
*/
.cmdbtn {
    background-image: url(cmdbtns.png);
    background-repeat: no-repeat;
    display: inline-block;
    width: 32px;
    height: 32px;
}

.cmdbtn-icon0 {
    background-position: 0 0;
}

.cmdbtn-icon1 {
    background-position: -32px 0;
}

.cmdbtn-icon10 {
    background-position: -64px 0;
}

.cmdbtn-icon100 {
    background-position: -96px 0;
}

.cmdbtn-icon101 {
    background-position: -128px 0;
}

.cmdbtn-icon102 {
    background-position: -160px 0;
}

.cmdbtn-icon103 {
    background-position: -192px 0;
}

.cmdbtn-icon104 {
    background-position: -224px 0;
}

.cmdbtn-icon105 {
    background-position: -256px 0;
}

.cmdbtn-icon106 {
    background-position: -288px 0;
}

.cmdbtn-icon107 {
    background-position: -320px 0;
}

.cmdbtn-icon108 {
    background-position: -352px 0;
}

.cmdbtn-icon109 {
    background-position: -384px 0;
}

.cmdbtn-icon11 {
    background-position: -416px 0;
}

.cmdbtn-icon110 {
    background-position: -448px 0;
}

.cmdbtn-icon111 {
    background-position: -480px 0;
}

.cmdbtn-icon112 {
    background-position: -512px 0;
}

.cmdbtn-icon113 {
    background-position: -544px 0;
}

.cmdbtn-icon114 {
    background-position: -576px 0;
}

.cmdbtn-icon115 {
    background-position: -608px 0;
}

.cmdbtn-icon116 {
    background-position: -640px 0;
}

.cmdbtn-icon117 {
    background-position: -672px 0;
}

.cmdbtn-icon118 {
    background-position: -704px 0;
}

.cmdbtn-icon119 {
    background-position: -736px 0;
}

.cmdbtn-icon12 {
    background-position: -768px 0;
}

.cmdbtn-icon120 {
    background-position: -800px 0;
}

.cmdbtn-icon121 {
    background-position: -832px 0;
}

.cmdbtn-icon122 {
    background-position: -864px 0;
}

.cmdbtn-icon123 {
    background-position: -896px 0;
}

.cmdbtn-icon124 {
    background-position: -928px 0;
}

.cmdbtn-icon125 {
    background-position: -960px 0;
}

.cmdbtn-icon126 {
    background-position: -992px 0;
}

.cmdbtn-icon127 {
    background-position: -1024px 0;
}

.cmdbtn-icon128 {
    background-position: -1056px 0;
}

.cmdbtn-icon129 {
    background-position: -1088px 0;
}

.cmdbtn-icon13 {
    background-position: -1120px 0;
}

.cmdbtn-icon130 {
    background-position: -1152px 0;
}

.cmdbtn-icon131 {
    background-position: -1184px 0;
}

.cmdbtn-icon132 {
    background-position: -1216px 0;
}

.cmdbtn-icon133 {
    background-position: -1248px 0;
}

.cmdbtn-icon134 {
    background-position: -1280px 0;
}

.cmdbtn-icon135 {
    background-position: -1312px 0;
}

.cmdbtn-icon136 {
    background-position: -1344px 0;
}

.cmdbtn-icon137 {
    background-position: -1376px 0;
}

.cmdbtn-icon138 {
    background-position: -1408px 0;
}

.cmdbtn-icon139 {
    background-position: -1440px 0;
}

.cmdbtn-icon14 {
    background-position: -1472px 0;
}

.cmdbtn-icon140 {
    background-position: -1504px 0;
}

.cmdbtn-icon141 {
    background-position: -1536px 0;
}

.cmdbtn-icon142 {
    background-position: -1568px 0;
}

.cmdbtn-icon143 {
    background-position: -1600px 0;
}

.cmdbtn-icon144 {
    background-position: -1632px 0;
}

.cmdbtn-icon145 {
    background-position: -1664px 0;
}

.cmdbtn-icon146 {
    background-position: -1696px 0;
}

.cmdbtn-icon147 {
    background-position: -1728px 0;
}

.cmdbtn-icon148 {
    background-position: -1760px 0;
}

.cmdbtn-icon149 {
    background-position: -1792px 0;
}

.cmdbtn-icon15 {
    background-position: -1824px 0;
}

.cmdbtn-icon150 {
    background-position: -1856px 0;
}

.cmdbtn-icon151 {
    background-position: -1888px 0;
}

.cmdbtn-icon152 {
    background-position: -1920px 0;
}

.cmdbtn-icon153 {
    background-position: -1952px 0;
}

.cmdbtn-icon154 {
    background-position: -1984px 0;
}

.cmdbtn-icon155 {
    background-position: -2016px 0;
}

.cmdbtn-icon156 {
    background-position: -2048px 0;
}

.cmdbtn-icon157 {
    background-position: -2080px 0;
}

.cmdbtn-icon158 {
    background-position: -2112px 0;
}

.cmdbtn-icon159 {
    background-position: -2144px 0;
}

.cmdbtn-icon16 {
    background-position: -2176px 0;
}

.cmdbtn-icon160 {
    background-position: -2208px 0;
}

.cmdbtn-icon161 {
    background-position: -2240px 0;
}

.cmdbtn-icon162 {
    background-position: -2272px 0;
}

.cmdbtn-icon163 {
    background-position: -2304px 0;
}

.cmdbtn-icon164 {
    background-position: -2336px 0;
}

.cmdbtn-icon165 {
    background-position: -2368px 0;
}

.cmdbtn-icon166 {
    background-position: -2400px 0;
}

.cmdbtn-icon167 {
    background-position: -2432px 0;
}

.cmdbtn-icon168 {
    background-position: -2464px 0;
}

.cmdbtn-icon169 {
    background-position: -2496px 0;
}

.cmdbtn-icon17 {
    background-position: -2528px 0;
}

.cmdbtn-icon170 {
    background-position: -2560px 0;
}

.cmdbtn-icon171 {
    background-position: -2592px 0;
}

.cmdbtn-icon172 {
    background-position: -2624px 0;
}

.cmdbtn-icon173 {
    background-position: -2656px 0;
}

.cmdbtn-icon174 {
    background-position: -2688px 0;
}

.cmdbtn-icon175 {
    background-position: -2720px 0;
}

.cmdbtn-icon176 {
    background-position: -2752px 0;
}

.cmdbtn-icon177 {
    background-position: -2784px 0;
}

.cmdbtn-icon178 {
    background-position: -2816px 0;
}

.cmdbtn-icon179 {
    background-position: -2848px 0;
}

.cmdbtn-icon18 {
    background-position: -2880px 0;
}

.cmdbtn-icon180 {
    background-position: -2912px 0;
}

.cmdbtn-icon181 {
    background-position: -2944px 0;
}

.cmdbtn-icon182 {
    background-position: -2976px 0;
}

.cmdbtn-icon183 {
    background-position: -3008px 0;
}

.cmdbtn-icon184 {
    background-position: -3040px 0;
}

.cmdbtn-icon185 {
    background-position: -3072px 0;
}

.cmdbtn-icon186 {
    background-position: -3104px 0;
}

.cmdbtn-icon187 {
    background-position: -3136px 0;
}

.cmdbtn-icon188 {
    background-position: -3168px 0;
}

.cmdbtn-icon189 {
    background-position: -3200px 0;
}

.cmdbtn-icon19 {
    background-position: -3232px 0;
}

.cmdbtn-icon190 {
    background-position: -3264px 0;
}

.cmdbtn-icon191 {
    background-position: -3296px 0;
}

.cmdbtn-icon192 {
    background-position: -3328px 0;
}

.cmdbtn-icon193 {
    background-position: -3360px 0;
}

.cmdbtn-icon194 {
    background-position: -3392px 0;
}

.cmdbtn-icon195 {
    background-position: -3424px 0;
}

.cmdbtn-icon196 {
    background-position: -3456px 0;
}

.cmdbtn-icon197 {
    background-position: -3488px 0;
}

.cmdbtn-icon198 {
    background-position: -3520px 0;
}

.cmdbtn-icon199 {
    background-position: -3552px 0;
}

.cmdbtn-icon2 {
    background-position: -3584px 0;
}

.cmdbtn-icon20 {
    background-position: -3616px 0;
}

.cmdbtn-icon200 {
    background-position: -3648px 0;
}

.cmdbtn-icon201 {
    background-position: -3680px 0;
}

.cmdbtn-icon202 {
    background-position: -3712px 0;
}

.cmdbtn-icon203 {
    background-position: -3744px 0;
}

.cmdbtn-icon204 {
    background-position: -3776px 0;
}

.cmdbtn-icon205 {
    background-position: -3808px 0;
}

.cmdbtn-icon206 {
    background-position: -3840px 0;
}

.cmdbtn-icon207 {
    background-position: -3872px 0;
}

.cmdbtn-icon208 {
    background-position: -3904px 0;
}

.cmdbtn-icon209 {
    background-position: -3936px 0;
}

.cmdbtn-icon21 {
    background-position: -3968px 0;
}

.cmdbtn-icon210 {
    background-position: -4000px 0;
}

.cmdbtn-icon211 {
    background-position: -4032px 0;
}

.cmdbtn-icon212 {
    background-position: -4064px 0;
}

.cmdbtn-icon213 {
    background-position: -4096px 0;
}

.cmdbtn-icon214 {
    background-position: -4128px 0;
}

.cmdbtn-icon215 {
    background-position: -4160px 0;
}

.cmdbtn-icon216 {
    background-position: -4192px 0;
}

.cmdbtn-icon217 {
    background-position: -4224px 0;
}

.cmdbtn-icon218 {
    background-position: -4256px 0;
}

.cmdbtn-icon219 {
    background-position: -4288px 0;
}

.cmdbtn-icon22 {
    background-position: -4320px 0;
}

.cmdbtn-icon220 {
    background-position: -4352px 0;
}

.cmdbtn-icon221 {
    background-position: -4384px 0;
}

.cmdbtn-icon222 {
    background-position: -4416px 0;
}

.cmdbtn-icon223 {
    background-position: -4448px 0;
}

.cmdbtn-icon224 {
    background-position: -4480px 0;
}

.cmdbtn-icon225 {
    background-position: -4512px 0;
}

.cmdbtn-icon226 {
    background-position: -4544px 0;
}

.cmdbtn-icon227 {
    background-position: -4576px 0;
}

.cmdbtn-icon228 {
    background-position: -4608px 0;
}

.cmdbtn-icon229 {
    background-position: -4640px 0;
}

.cmdbtn-icon23 {
    background-position: -4672px 0;
}

.cmdbtn-icon230 {
    background-position: -4704px 0;
}

.cmdbtn-icon231 {
    background-position: -4736px 0;
}

.cmdbtn-icon232 {
    background-position: -4768px 0;
}

.cmdbtn-icon233 {
    background-position: -4800px 0;
}

.cmdbtn-icon234 {
    background-position: -4832px 0;
}

.cmdbtn-icon235 {
    background-position: -4864px 0;
}

.cmdbtn-icon236 {
    background-position: -4896px 0;
}

.cmdbtn-icon237 {
    background-position: -4928px 0;
}

.cmdbtn-icon238 {
    background-position: -4960px 0;
}

.cmdbtn-icon239 {
    background-position: -4992px 0;
}

.cmdbtn-icon24 {
    background-position: -5024px 0;
}

.cmdbtn-icon240 {
    background-position: -5056px 0;
}

.cmdbtn-icon241 {
    background-position: -5088px 0;
}

.cmdbtn-icon242 {
    background-position: -5120px 0;
}

.cmdbtn-icon243 {
    background-position: -5152px 0;
}

.cmdbtn-icon244 {
    background-position: -5184px 0;
}

.cmdbtn-icon245 {
    background-position: -5216px 0;
}

.cmdbtn-icon246 {
    background-position: -5248px 0;
}

.cmdbtn-icon247 {
    background-position: -5280px 0;
}

.cmdbtn-icon248 {
    background-position: -5312px 0;
}

.cmdbtn-icon249 {
    background-position: -5344px 0;
}

.cmdbtn-icon25 {
    background-position: -5376px 0;
}

.cmdbtn-icon250 {
    background-position: -5408px 0;
}

.cmdbtn-icon251 {
    background-position: -5440px 0;
}

.cmdbtn-icon252 {
    background-position: -5472px 0;
}

.cmdbtn-icon253 {
    background-position: -5504px 0;
}

.cmdbtn-icon254 {
    background-position: -5536px 0;
}

.cmdbtn-icon255 {
    background-position: -5568px 0;
}

.cmdbtn-icon256 {
    background-position: -5600px 0;
}

.cmdbtn-icon257 {
    background-position: -5632px 0;
}

.cmdbtn-icon258 {
    background-position: -5664px 0;
}

.cmdbtn-icon259 {
    background-position: -5696px 0;
}

.cmdbtn-icon26 {
    background-position: -5728px 0;
}

.cmdbtn-icon260 {
    background-position: -5760px 0;
}

.cmdbtn-icon261 {
    background-position: -5792px 0;
}

.cmdbtn-icon262 {
    background-position: -5824px 0;
}

.cmdbtn-icon263 {
    background-position: -5856px 0;
}

.cmdbtn-icon264 {
    background-position: -5888px 0;
}

.cmdbtn-icon265 {
    background-position: -5920px 0;
}

.cmdbtn-icon266 {
    background-position: -5952px 0;
}

.cmdbtn-icon267 {
    background-position: -5984px 0;
}

.cmdbtn-icon268 {
    background-position: -6016px 0;
}

.cmdbtn-icon269 {
    background-position: -6048px 0;
}

.cmdbtn-icon27 {
    background-position: -6080px 0;
}

.cmdbtn-icon270 {
    background-position: -6112px 0;
}

.cmdbtn-icon271 {
    background-position: -6144px 0;
}

.cmdbtn-icon272 {
    background-position: -6176px 0;
}

.cmdbtn-icon273 {
    background-position: -6208px 0;
}

.cmdbtn-icon274 {
    background-position: -6240px 0;
}

.cmdbtn-icon275 {
    background-position: -6272px 0;
}

.cmdbtn-icon276 {
    background-position: -6304px 0;
}

.cmdbtn-icon277 {
    background-position: -6336px 0;
}

.cmdbtn-icon278 {
    background-position: -6368px 0;
}

.cmdbtn-icon279 {
    background-position: -6400px 0;
}

.cmdbtn-icon28 {
    background-position: -6432px 0;
}

.cmdbtn-icon280 {
    background-position: -6464px 0;
}

.cmdbtn-icon281 {
    background-position: -6496px 0;
}

.cmdbtn-icon282 {
    background-position: -6528px 0;
}

.cmdbtn-icon283 {
    background-position: -6560px 0;
}

.cmdbtn-icon284 {
    background-position: -6592px 0;
}

.cmdbtn-icon285 {
    background-position: -6624px 0;
}

.cmdbtn-icon286 {
    background-position: -6656px 0;
}

.cmdbtn-icon287 {
    background-position: -6688px 0;
}

.cmdbtn-icon288 {
    background-position: -6720px 0;
}

.cmdbtn-icon289 {
    background-position: -6752px 0;
}

.cmdbtn-icon29 {
    background-position: -6784px 0;
}

.cmdbtn-icon290 {
    background-position: -6816px 0;
}

.cmdbtn-icon291 {
    background-position: -6848px 0;
}

.cmdbtn-icon292 {
    background-position: -6880px 0;
}

.cmdbtn-icon293 {
    background-position: -6912px 0;
}

.cmdbtn-icon294 {
    background-position: -6944px 0;
}

.cmdbtn-icon295 {
    background-position: -6976px 0;
}

.cmdbtn-icon296 {
    background-position: -7008px 0;
}

.cmdbtn-icon297 {
    background-position: -7040px 0;
}

.cmdbtn-icon298 {
    background-position: -7072px 0;
}

.cmdbtn-icon299 {
    background-position: -7104px 0;
}

.cmdbtn-icon3 {
    background-position: -7136px 0;
}

.cmdbtn-icon30 {
    background-position: -7168px 0;
}

.cmdbtn-icon300 {
    background-position: -7200px 0;
}

.cmdbtn-icon301 {
    background-position: -7232px 0;
}

.cmdbtn-icon302 {
    background-position: -7264px 0;
}

.cmdbtn-icon303 {
    background-position: -7296px 0;
}

.cmdbtn-icon304 {
    background-position: -7328px 0;
}

.cmdbtn-icon305 {
    background-position: -7360px 0;
}

.cmdbtn-icon306 {
    background-position: -7392px 0;
}

.cmdbtn-icon307 {
    background-position: -7424px 0;
}

.cmdbtn-icon308 {
    background-position: -7456px 0;
}

.cmdbtn-icon309 {
    background-position: -7488px 0;
}

.cmdbtn-icon31 {
    background-position: -7520px 0;
}

.cmdbtn-icon310 {
    background-position: -7552px 0;
}

.cmdbtn-icon311 {
    background-position: -7584px 0;
}

.cmdbtn-icon312 {
    background-position: -7616px 0;
}

.cmdbtn-icon313 {
    background-position: -7648px 0;
}

.cmdbtn-icon314 {
    background-position: -7680px 0;
}

.cmdbtn-icon315 {
    background-position: -7712px 0;
}

.cmdbtn-icon316 {
    background-position: -7744px 0;
}

.cmdbtn-icon317 {
    background-position: -7776px 0;
}

.cmdbtn-icon318 {
    background-position: -7808px 0;
}

.cmdbtn-icon319 {
    background-position: -7840px 0;
}

.cmdbtn-icon32 {
    background-position: -7872px 0;
}

.cmdbtn-icon320 {
    background-position: -7904px 0;
}

.cmdbtn-icon321 {
    background-position: -7936px 0;
}

.cmdbtn-icon322 {
    background-position: -7968px 0;
}

.cmdbtn-icon323 {
    background-position: -8000px 0;
}

.cmdbtn-icon324 {
    background-position: -8032px 0;
}

.cmdbtn-icon325 {
    background-position: -8064px 0;
}

.cmdbtn-icon326 {
    background-position: -8096px 0;
}

.cmdbtn-icon327 {
    background-position: -8128px 0;
}

.cmdbtn-icon328 {
    background-position: -8160px 0;
}

.cmdbtn-icon329 {
    background-position: -8192px 0;
}

.cmdbtn-icon33 {
    background-position: -8224px 0;
}

.cmdbtn-icon330 {
    background-position: -8256px 0;
}

.cmdbtn-icon331 {
    background-position: -8288px 0;
}

.cmdbtn-icon332 {
    background-position: -8320px 0;
}

.cmdbtn-icon333 {
    background-position: -8352px 0;
}

.cmdbtn-icon334 {
    background-position: -8384px 0;
}

.cmdbtn-icon335 {
    background-position: -8416px 0;
}

.cmdbtn-icon336 {
    background-position: -8448px 0;
}

.cmdbtn-icon337 {
    background-position: -8480px 0;
}

.cmdbtn-icon338 {
    background-position: -8512px 0;
}

.cmdbtn-icon339 {
    background-position: -8544px 0;
}

.cmdbtn-icon34 {
    background-position: -8576px 0;
}

.cmdbtn-icon340 {
    background-position: -8608px 0;
}

.cmdbtn-icon341 {
    background-position: -8640px 0;
}

.cmdbtn-icon342 {
    background-position: -8672px 0;
}

.cmdbtn-icon343 {
    background-position: -8704px 0;
}

.cmdbtn-icon344 {
    background-position: -8736px 0;
}

.cmdbtn-icon345 {
    background-position: -8768px 0;
}

.cmdbtn-icon346 {
    background-position: -8800px 0;
}

.cmdbtn-icon347 {
    background-position: -8832px 0;
}

.cmdbtn-icon348 {
    background-position: -8864px 0;
}

.cmdbtn-icon349 {
    background-position: -8896px 0;
}

.cmdbtn-icon35 {
    background-position: -8928px 0;
}

.cmdbtn-icon350 {
    background-position: -8960px 0;
}

.cmdbtn-icon351 {
    background-position: -8992px 0;
}

.cmdbtn-icon352 {
    background-position: -9024px 0;
}

.cmdbtn-icon353 {
    background-position: -9056px 0;
}

.cmdbtn-icon354 {
    background-position: -9088px 0;
}

.cmdbtn-icon355 {
    background-position: -9120px 0;
}

.cmdbtn-icon356 {
    background-position: -9152px 0;
}

.cmdbtn-icon357 {
    background-position: -9184px 0;
}

.cmdbtn-icon358 {
    background-position: -9216px 0;
}

.cmdbtn-icon359 {
    background-position: -9248px 0;
}

.cmdbtn-icon36 {
    background-position: -9280px 0;
}

.cmdbtn-icon360 {
    background-position: -9312px 0;
}

.cmdbtn-icon361 {
    background-position: -9344px 0;
}

.cmdbtn-icon362 {
    background-position: -9376px 0;
}

.cmdbtn-icon363 {
    background-position: -9408px 0;
}

.cmdbtn-icon364 {
    background-position: -9440px 0;
}

.cmdbtn-icon365 {
    background-position: -9472px 0;
}

.cmdbtn-icon366 {
    background-position: -9504px 0;
}

.cmdbtn-icon367 {
    background-position: -9536px 0;
}

.cmdbtn-icon368 {
    background-position: -9568px 0;
}

.cmdbtn-icon369 {
    background-position: -9600px 0;
}

.cmdbtn-icon37 {
    background-position: -9632px 0;
}

.cmdbtn-icon370 {
    background-position: -9664px 0;
}

.cmdbtn-icon371 {
    background-position: -9696px 0;
}

.cmdbtn-icon372 {
    background-position: -9728px 0;
}

.cmdbtn-icon373 {
    background-position: -9760px 0;
}

.cmdbtn-icon374 {
    background-position: -9792px 0;
}

.cmdbtn-icon375 {
    background-position: -9824px 0;
}

.cmdbtn-icon376 {
    background-position: -9856px 0;
}

.cmdbtn-icon377 {
    background-position: -9888px 0;
}

.cmdbtn-icon378 {
    background-position: -9920px 0;
}

.cmdbtn-icon379 {
    background-position: -9952px 0;
}

.cmdbtn-icon38 {
    background-position: -9984px 0;
}

.cmdbtn-icon380 {
    background-position: -10016px 0;
}

.cmdbtn-icon381 {
    background-position: -10048px 0;
}

.cmdbtn-icon382 {
    background-position: -10080px 0;
}

.cmdbtn-icon383 {
    background-position: -10112px 0;
}

.cmdbtn-icon384 {
    background-position: -10144px 0;
}

.cmdbtn-icon385 {
    background-position: -10176px 0;
}

.cmdbtn-icon386 {
    background-position: -10208px 0;
}

.cmdbtn-icon387 {
    background-position: -10240px 0;
}

.cmdbtn-icon388 {
    background-position: -10272px 0;
}

.cmdbtn-icon389 {
    background-position: -10304px 0;
}

.cmdbtn-icon39 {
    background-position: -10336px 0;
}

.cmdbtn-icon4 {
    background-position: -10368px 0;
}

.cmdbtn-icon40 {
    background-position: -10400px 0;
}

.cmdbtn-icon41 {
    background-position: -10432px 0;
}

.cmdbtn-icon42 {
    background-position: -10464px 0;
}

.cmdbtn-icon43 {
    background-position: -10496px 0;
}

.cmdbtn-icon44 {
    background-position: -10528px 0;
}

.cmdbtn-icon45 {
    background-position: -10560px 0;
}

.cmdbtn-icon46 {
    background-position: -10592px 0;
}

.cmdbtn-icon47 {
    background-position: -10624px 0;
}

.cmdbtn-icon48 {
    background-position: -10656px 0;
}

.cmdbtn-icon49 {
    background-position: -10688px 0;
}

.cmdbtn-icon5 {
    background-position: -10720px 0;
}

.cmdbtn-icon50 {
    background-position: -10752px 0;
}

.cmdbtn-icon51 {
    background-position: -10784px 0;
}

.cmdbtn-icon52 {
    background-position: -10816px 0;
}

.cmdbtn-icon53 {
    background-position: -10848px 0;
}

.cmdbtn-icon54 {
    background-position: -10880px 0;
}

.cmdbtn-icon55 {
    background-position: -10912px 0;
}

.cmdbtn-icon56 {
    background-position: -10944px 0;
}

.cmdbtn-icon57 {
    background-position: -10976px 0;
}

.cmdbtn-icon58 {
    background-position: -11008px 0;
}

.cmdbtn-icon59 {
    background-position: -11040px 0;
}

.cmdbtn-icon6 {
    background-position: -11072px 0;
}

.cmdbtn-icon60 {
    background-position: -11104px 0;
}

.cmdbtn-icon61 {
    background-position: -11136px 0;
}

.cmdbtn-icon62 {
    background-position: -11168px 0;
}

.cmdbtn-icon63 {
    background-position: -11200px 0;
}

.cmdbtn-icon64 {
    background-position: -11232px 0;
}

.cmdbtn-icon65 {
    background-position: -11264px 0;
}

.cmdbtn-icon66 {
    background-position: -11296px 0;
}

.cmdbtn-icon67 {
    background-position: -11328px 0;
}

.cmdbtn-icon68 {
    background-position: -11360px 0;
}

.cmdbtn-icon69 {
    background-position: -11392px 0;
}

.cmdbtn-icon7 {
    background-position: -11424px 0;
}

.cmdbtn-icon70 {
    background-position: -11456px 0;
}

.cmdbtn-icon71 {
    background-position: -11488px 0;
}

.cmdbtn-icon72 {
    background-position: -11520px 0;
}

.cmdbtn-icon73 {
    background-position: -11552px 0;
}

.cmdbtn-icon74 {
    background-position: -11584px 0;
}

.cmdbtn-icon75 {
    background-position: -11616px 0;
}

.cmdbtn-icon76 {
    background-position: -11648px 0;
}

.cmdbtn-icon77 {
    background-position: -11680px 0;
}

.cmdbtn-icon78 {
    background-position: -11712px 0;
}

.cmdbtn-icon79 {
    background-position: -11744px 0;
}

.cmdbtn-icon8 {
    background-position: -11776px 0;
}

.cmdbtn-icon80 {
    background-position: -11808px 0;
}

.cmdbtn-icon81 {
    background-position: -11840px 0;
}

.cmdbtn-icon82 {
    background-position: -11872px 0;
}

.cmdbtn-icon83 {
    background-position: -11904px 0;
}

.cmdbtn-icon84 {
    background-position: -11936px 0;
}

.cmdbtn-icon85 {
    background-position: -11968px 0;
}

.cmdbtn-icon86 {
    background-position: -12000px 0;
}

.cmdbtn-icon87 {
    background-position: -12032px 0;
}

.cmdbtn-icon88 {
    background-position: -12064px 0;
}

.cmdbtn-icon89 {
    background-position: -12096px 0;
}

.cmdbtn-icon9 {
    background-position: -12128px 0;
}

.cmdbtn-icon90 {
    background-position: -12160px 0;
}

.cmdbtn-icon91 {
    background-position: -12192px 0;
}

.cmdbtn-icon92 {
    background-position: -12224px 0;
}

.cmdbtn-icon93 {
    background-position: -12256px 0;
}

.cmdbtn-icon94 {
    background-position: -12288px 0;
}

.cmdbtn-icon95 {
    background-position: -12320px 0;
}

.cmdbtn-icon96 {
    background-position: -12352px 0;
}

.cmdbtn-icon97 {
    background-position: -12384px 0;
}

.cmdbtn-icon98 {
    background-position: -12416px 0;
}

.cmdbtn-icon99 {
    background-position: -12448px 0;
}

/* ==================================================================================================================
tticons.css
==================================================================================================================
*/
.tticon {
    background-image: url(tticons.png);
    background-repeat: no-repeat;
    display: inline-block;
    width: 14px;
    height: 14px;
}

.tticon-energy {
    background-position: 0 0;
}

.tticon-gas0 {
    background-position: -14px 0;
}

.tticon-gas1 {
    background-position: -28px 0;
}

.tticon-gas2 {
    background-position: -42px 0;
}

.tticon-ore {
    background-position: -56px 0;
}

.tticon-supply0 {
    background-position: -70px 0;
}

.tticon-supply1 {
    background-position: -84px 0;
}

.tticon-supply2 {
    background-position: -98px 0;
}
