//{{NO_DEPENDENCIES}}
// Microsoft Visual C++ generated include file.
// Used by UI.rc
//
#define IDD_DIALOG1                     101
#define IDD_SETTINGS                    101
#define IDC_EDITIP                      1001
#define IDC_EDITPORT                    1003
#define IDC_EDITLPORT                   1004
#define IDC_BUTTON1                     1005
#define IDC_STATUS                      1006

// Next default values for new objects
// 
#ifdef APSTUDIO_INVOKED
#ifndef APSTUDIO_READONLY_SYMBOLS
#define _APS_NEXT_RESOURCE_VALUE        102
#define _APS_NEXT_COMMAND_VALUE         40001
#define _APS_NEXT_CONTROL_VALUE         1007
#define _APS_NEXT_SYMED_VALUE           101
#endif
#endif
