#pragma once

void showSettingsDialog();
void hideSettingsDialog();

const char* getHostIPString();
const char* getHostPortString();
const char* getLocalPortString();
void setStatusString(const char *statusText);
