using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace BWMemoryEdit.Enums
{
    public enum FlingyType : ushort
    {
        Scourge,
        Broodling,
        Infested_Terran,
        Guardian_Cocoon,
        Defiler,
        Drone,
        Egg,
        Guardian,
        Hydralisk,
        Infested_Kerrigan,
        Larva,
        Mutalisk,
        Overlord,
        Queen,
        Ultralisk,
        Zergling,
        Cerebrate,
        Infested_Command_Center,
        Spawning_Pool,
        Mature_Chrysalis,
        Evolution_Chamber,
        Creep_Colony,
        Hatchery,
        Hive,
        Lair,
        Sunken_Colony,
        Greater_Spire,
        Defiler_Mound,
        Queens_Nest,
        Nydus_Canal,
        Overmind_With_Shell,
        Overmind_Without_Shell,
        Ultralisk_Cavern,
        Extractor,
        Hydralisk_Den,
        Spire,
        Spore_Colony,
        Arbiter,
        Archon_Energy,
        Carrier,
        Dragoon,
        Interceptor,
        Probe,
        Scout,
        Shuttle,
        High_Templar,
        Dark_Templar_Hero,
        Reaver,
        Scarab,
        Zealot,
        Observer,
        Templar_Archives,
        Assimilator,
        Observatory,
        Citadel_of_Adun,
        Forge,
        Gateway,
        Cybernetics_Core,
        Khaydarin_Crystal_Formation,
        Nexus,
        Photon_Cannon,
        Arbiter_Tribunal,
        Pylon,
        Robotics_Facility,
        Shield_Battery,
        Stargate,
        Stasis_Cell_Prison,
        Robotics_Support_Bay,
        Protoss_Temple,
        Fleet_Beacon,
        Battlecruiser,
        Civilian,
        Dropship,
        Firebat,
        Ghost,
        Goliath_Base,
        Goliath_Turret,
        Sarah_Kerrigan,
        Marine,
        Scanner_Sweep,
        Wraith,
        SCV,
        Siege_Tank_Tank_Base,
        Siege_Tank_Tank_Turret,
        Siege_Tank_Siege_Base,
        Siege_Tank_Siege_Turret,
        Science_Vessel_Base,
        Science_Vessel_Turret,
        Vulture,
        Spider_Mine,
        Terran_Academy,
        Barracks,
        Armory,
        Comsat_Station,
        Command_Center,
        Supply_Depot,
        Control_Tower,
        Factory,
        Covert_Ops,
        Ion_Cannon,
        Machine_Shop,
        Missile_Turret_Base,
        Crashed_Battlecruiser,
        Physics_Lab,
        Bunker,
        Refinery,
        Immobile_Barracks,
        Science_Facility,
        Nuke_Silo,
        Nuclear_Missile,
        Starport,
        Engineering_Bay,
        Terran_Construction_Large,
        Terran_Construction_Small,
        Ragnasaur_Ashworld,
        Rhynadon_Badlands,
        Bengalaas_Jungle,
        Vespene_Geyser,
        Mineral_Field_Type1,
        Mineral_Field_Type2,
        Mineral_Field_Type3,
        Independent_Starport_Unused,
        Zerg_Beacon,
        Terran_Beacon,
        Protoss_Beacon,
        Dark_Swarm,
        Flag,
        Young_Chrysalis,
        Psi_Emitter,
        Data_Disc,
        Khaydarin_Crystal,
        Mineral_Chunk_Type1,
        Mineral_Chunk_Type2,
        Protoss_Gas_Orb_Type1,
        Protoss_Gas_Orb_Type2,
        Zerg_Gas_Sac_Type1,
        Zerg_Gas_Sac_Type2,
        Terran_Gas_Tank_Type1,
        Terran_Gas_Tank_Type2,
        Map_Revealer,
        Start_Location,
        Fusion_Cutter_Hit,
        Gauss_Rifle_Hit,
        C_10_Canister_Rifle_Hit,
        Gemini_Missiles,
        Fragmentation_Grenade,
        Lockdown_LongBolt_Hellfire_Missile,
        Unknown147,
        ATS_ATA_Laser_Battery,
        Burst_Lasers,
        Arclite_Shock_Cannon_Hit,
        EMP_Missile,
        Dual_Photon_Blasters_Hit,
        Particle_Beam_Hit,
        Anti_Matter_Missile,
        Pulse_Cannon,
        Psionic_Shockwave_Hit,
        Psionic_Storm,
        Yamato_Gun,
        Phase_Disruptor,
        STA_STS_Cannon_Overlay,
        Sunken_Colony_Tentacle,
        Venom_Unused_Zerg_Weapon,
        Acid_Spore,
        Unknown164,
        Glave_Wurm,
        Seeker_Spores,
        Queen_Spell_Carrier,
        Plague_Cloud,
        Consume,
        Ensnare,
        Needle_Spine_Hit,
        White_Circle_Invisible,
        Left_Upper_Level_Door,
        Right_Upper_Level_Door,
        Substructure_Left_Door,
        Substructure_Right_Door,
        Substructure_Opening_Hole,
        Floor_Gun_Trap,
        Floor_Missile_Trap,
        Wall_Missile_Trap,
        Wall_Missile_Trap2,
        Wall_Flame_Trap,
        Wall_Flame_Trap2,
        Lurker_Egg,
        Devourer,
        Lurker,
        Dark_Archon_Energy,
        Dark_Templar_Unit,
        Medic,
        Valkyrie,
        Corsair,
        Disruption_Web,
        Overmind_Cocoon,
        Psi_Disrupter,
        Warp_Gate,
        Power_Generator,
        XelNaga_Temple,
        Scantid_Desert,
        Kakaru_Twilight,
        Ursadon_Ice,
        Optical_Flare_Grenade,
        Halo_Rockets,
        Subterranean_Spines,
        Corrosive_Acid_Shot,
        Corrosive_Acid_Hit,
        Neutron_Flare,
        Uraj,
        Khalis
    }
}
